package filegateway

import (
	"context"
	"io/ioutil"
	"os"
	"testing"

	"cloud.google.com/go/storage"
	"github.com/fsouza/fake-gcs-server/fakestorage"
	"github.com/stretchr/testify/require"
	"google.golang.org/api/option"
)

func tryGCSConnect(t *testing.T, hostport string) error {
	client, err := storage.NewClient(
		context.TODO(),
		option.WithEndpoint("http://"+hostport+"/storage/v1/"),
		option.WithoutAuthentication(),
	)
	if err != nil {
		return err
	}
	require.NoError(t, client.Bucket("test").Create(context.TODO(), "", nil))

	o := client.Bucket("test").Object("toto")
	w := o.NewWriter(context.TODO())
	_, err = w.Write([]byte("coucou\n"))
	require.NoError(t, err)
	require.NoError(t, w.Close())

	r, err := o.NewReader(context.TODO())
	require.NoError(t, err)
	b, err := ioutil.ReadAll(r)
	require.NoError(t, err)

	require.Equal(t, "coucou\n", string(b))

	return nil
}

func TestGCSFilesystem(t *testing.T) {
	var (
		host    = "localhost"
		port    = "4443"
		portInt = uint16(4443)
	)

	server, err := fakestorage.NewServerWithOptions(fakestorage.Options{
		Scheme:     "http",
		Host:       host,
		Port:       portInt,
		PublicHost: host + ":" + port,
	})
	require.NoError(t, err)

	defer server.Stop()

	os.Setenv("STORAGE_EMULATOR_HOST", host+":"+port)
	defer os.Unsetenv("STORAGE_EMULATOR_HOST")

	require.NoError(t, tryGCSConnect(t, host+":"+port))

	FilesystemTestSuite(t, func(options FilesystemOptions) (Filesystem, error) {
		fs, err := gcsFilesystemFactory(options)
		if err != nil {
			return nil, err
		}
		return fs, nil
	}, FilesystemOptions{
		Scheme: "gcs",
		Host:   "test",
		Custom: CustomOptions{
			"endpoint":    "http://" + host + ":" + port + "/storage/v1/",
			"credentials": "noauth",
		},
	})
}
