package filegateway

import (
	"fmt"
	"log"
	"os"
	"os/exec"
	"strconv"
	"strings"
	"testing"
)

func IsInCI() bool {
	return os.Getenv("CI") == "true"
}

func dockerCompose(args ...string) error {
	args = append([]string{"-f", "docker/docker-compose.yaml"}, args...)
	cmd := exec.Command("docker-compose", args...)
	output, err := cmd.CombinedOutput()
	if err != nil {
		log.Printf("docker-compose %s output:\n%s", args, string(output))
	}
	return err
}

func mustDockerCompose(args ...string) {
	if err := dockerCompose(args...); err != nil {
		log.Fatalf("Error running docker-compose %s: %s", args, err)
	}
}

func dockerComposeOutput(args ...string) (string, error) {
	args = append([]string{"-f", "docker/docker-compose.yaml"}, args...)
	cmd := exec.Command("docker-compose", args...)
	output, err := cmd.CombinedOutput()
	return string(output), err
}

func dockerComposePort(service string, port string) (string, error) {
	output, err := dockerComposeOutput("port", service, port)
	if err != nil {
		log.Printf("docker-compose port %s %s output:\n%s", service, port, string(output))
		return "", err
	}
	output = strings.Trim(output, "\n")
	if !strings.Contains(output, ":") {
		return "", fmt.Errorf("docker-compose port: invalid output '%s'", output)
	}
	port = output[strings.LastIndex(output, ":")+1:]
	if _, err := strconv.Atoi(port); err != nil {
		return "", err
	}
	return port, nil
}

func TestMain(m *testing.M) {
	if !IsInCI() {
		log.Println("Starting containers...")
		mustDockerCompose("down")
		mustDockerCompose("up", "--no-start")
		mustDockerCompose("start")
		log.Println("Containers ready")
	}

	code := m.Run()

	os.Exit(code)
}
