package filegateway

import (
	"fmt"
	"os"
	"testing"

	"golang.org/x/crypto/ssh"
)

const hostPubkey = "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIL+BET9m3jCj5yhI1cRrHJF8HDG5ZMnCl/vI07sY+Oyr cdevienne@needle"
const privateKey = `-----BEGIN OPENSSH PRIVATE KEY-----
b3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAAAMwAAAAtzc2gtZW
QyNTUxOQAAACAshcT+ZFRn/TwKscgLPyQXZdR2SiwifECXNbhi6xQAfAAAAJhlUsyrZVLM
qwAAAAtzc2gtZWQyNTUxOQAAACAshcT+ZFRn/TwKscgLPyQXZdR2SiwifECXNbhi6xQAfA
AAAEB5hrPxkowlvYYk0uQVn6AjGpmHOYN3u5QFJwiTH9zHLiyFxP5kVGf9PAqxyAs/JBdl
1HZKLCJ8QJc1uGLrFAB8AAAAEGNkZXZpZW5uZUBuZWVkbGUBAgMEBQ==
-----END OPENSSH PRIVATE KEY-----
`

func pingScpServer(t *testing.T, hostport string) func() error {
	hostKey, _, _, _, err := ssh.ParseAuthorizedKey([]byte(hostPubkey))
	if err != nil {
		t.Fatal(err)
	}
	signer, err := ssh.ParsePrivateKey([]byte(privateKey))
	if err != nil {
		t.Fatal(err)
	}

	return func() error {
		conn, err := ssh.Dial("tcp", hostport, &ssh.ClientConfig{
			HostKeyCallback: ssh.FixedHostKey(hostKey),
			Auth:            []ssh.AuthMethod{ssh.PublicKeys(signer)},
			User:            "root",
		})
		if err == nil {
			conn.Close()
		} else {
			fmt.Println(err)
		}
		return err
	}
}

func TestScpFilesystem(t *testing.T) {
	var host, port string
	if IsInCI() {
		host = os.Getenv("SSHD_PORT_22_TCP_ADDR")
		port = os.Getenv("SSHD_PORT_22_TCP_PORT")
	} else {
		var err error
		port, err = dockerComposePort("scp", "22")
		if err != nil {
			t.Fatal(err)
		}
		host = "127.0.0.1"
	}

	hostport := host + ":" + port
	if err := pingScpServer(t, hostport)(); err != nil {
		t.Fatal("ping", hostport, err)
	}

	FilesystemTestSuite(t, func(options FilesystemOptions) (Filesystem, error) {
		fs, err := scpFilesystemFactory(options)
		if err != nil {
			return nil, err
		}
		return fs, nil
	}, FilesystemOptions{
		Host:     host,
		Port:     port,
		Username: "root",
		Custom: CustomOptions{
			"host-pubkey": hostPubkey,
			"private-key": privateKey,
		},
	})

}
