package routing

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"orus.io/xbus/xbus-stdlib/service"
	"orus.io/xbus/xbus-stdlib/service/svctest"
	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/envelope"
)

func TestDeleter(t *testing.T) {
	table := NewMemoryTableStore("test")

	tester := svctest.TestWorker(t, NewDeleterAsWorker,
		&api.Actor{Name: "bob"},
		service.Settings{
			"store.backend":  "memory",
			"store.name":     "test",
			"extract-key-jq": ".ref",
		},
	)

	t.Run("delete-entry", func(t *testing.T) {
		e := envelope.MustNewEnvelope(
			envelope.MustNewTextMessage("msg", `{"ref":"FILE001","agency":"a2"}`))

		require.NoError(t, table.Set("FILE001", "a1"))

		outputs, err := tester.Run(svctest.Inputs{api.DefaultInput: e})

		assert.NoError(t, err)

		assert.Equal(t, 1, len(outputs.Forwarded))
		assert.Equal(t, 0, len(outputs.Sent))

		if assert.Contains(t, outputs.Forwarded, "default") {
			assert.Equal(t, e.ID(), outputs.Forwarded["default"])
		}
		_, err = table.Get("FILE001")
		assert.EqualError(t, err, "no such key")
	})
}
