package routing

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"orus.io/xbus/xbus-stdlib/service"
	"orus.io/xbus/xbus-stdlib/service/svctest"
	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/envelope"
)

func TestUpdater(t *testing.T) {
	table := NewMemoryTableStore("test")

	tester := svctest.TestWorker(t, NewUpdaterAsWorker,
		&api.Actor{Name: "bob"},
		service.Settings{
			"store.backend":    "memory",
			"store.name":       "test",
			"extract-key-jq":   ".ref",
			"extract-value-jq": ".agency",
		},
	)

	t.Run("new-entry", func(t *testing.T) {
		e := envelope.MustNewEnvelope(
			envelope.MustNewTextMessage("msg", `{"ref":"FILE001","agency":"a1"}`))

		outputs, err := tester.Run(svctest.Inputs{api.DefaultInput: e})

		assert.NoError(t, err)

		assert.Equal(t, 1, len(outputs.Forwarded))
		assert.Equal(t, 0, len(outputs.Sent))

		if assert.Contains(t, outputs.Forwarded, "default") {
			assert.Equal(t, e.ID(), outputs.Forwarded["default"])
		}
		v, err := table.Get("FILE001")
		assert.NoError(t, err)
		assert.Equal(t, "a1", v)
	})

	t.Run("update-entry", func(t *testing.T) {
		e := envelope.MustNewEnvelope(
			envelope.MustNewTextMessage("msg", `{"ref":"FILE001","agency":"a2"}`))

		outputs, err := tester.Run(svctest.Inputs{api.DefaultInput: e})

		assert.NoError(t, err)

		assert.Equal(t, 1, len(outputs.Forwarded))
		assert.Equal(t, 0, len(outputs.Sent))

		if assert.Contains(t, outputs.Forwarded, "default") {
			assert.Equal(t, e.ID(), outputs.Forwarded["default"])
		}
		v, err := table.Get("FILE001")
		assert.NoError(t, err)
		assert.Equal(t, "a2", v)
	})
}
